@@ -2,10 +2,11 @@
 from __future__ import unicode_literals
 
 import sys
-import json
-from pathlib import Path
+import json as _builtin_json
+import gzip
 
-from .json import ujson
+from . import ujson
+from .util import force_path, force_string
 
 
 def json_dumps(data, indent=0, sort_keys=False):
@@ -17,7 +18,10 @@ def json_dumps(data, indent=0, sort_keys=False):
     RETURNS (unicode): The serialized string.
     """
     if sort_keys:
-        result = json.dumps(data, indent=indent, sort_keys=sort_keys)
+        indent = None if indent == 0 else indent
+        result = _builtin_json.dumps(
+            data, indent=indent, separators=(",", ":"), sort_keys=sort_keys
+        )
     else:
         result = ujson.dumps(data, indent=indent, escape_forward_slashes=False)
     if sys.version_info[0] == 2:  # Python 2
@@ -43,11 +47,22 @@ def read_json(location):
     if location == "-":  # reading from sys.stdin
         data = sys.stdin.read()
         return ujson.loads(data)
-    file_path = _force_path(location)
+    file_path = force_path(location)
     with file_path.open("r", encoding="utf8") as f:
         return ujson.load(f)
 
 
+def read_gzip_json(location):
+    """Load JSON from a gzipped file.
+
+        location (unicode / Path): The file path.
+        RETURNS (dict / list): The loaded JSON content.
+    """
+    file_path = force_string(location)
+    with gzip.open(file_path, "r") as f:
+        return ujson.load(f)
+
+
 def write_json(location, data, indent=2):
     """Create a .json file and dump contents or write to standard
     output.
@@ -60,11 +75,24 @@ def write_json(location, data, indent=2):
     if location == "-":  # writing to stdout
         print(json_data)
     else:
-        file_path = _force_path(location, require_exists=False)
+        file_path = force_path(location, require_exists=False)
         with file_path.open("w", encoding="utf8") as f:
             f.write(json_data)
 
 
+def write_gzip_json(location, data, indent=2):
+    """Create a .json.gz file and dump contents.
+
+    location (unicode / Path): The file path.
+    data: The JSON-serializable data to output.
+    indent (int): Number of spaces used to indent JSON.
+    """
+    json_data = json_dumps(data, indent=indent)
+    file_path = force_string(location)
+    with gzip.open(file_path, "w") as f:
+        f.write(json_data.encode("utf-8"))
+
+
 def read_jsonl(location, skip=False):
     """Read a .jsonl file or standard input and yield contents line by line.
     Blank lines will always be skipped.
@@ -77,24 +105,30 @@ def read_jsonl(location, skip=False):
         for line in _yield_json_lines(sys.stdin, skip=skip):
             yield line
     else:
-        file_path = _force_path(location)
+        file_path = force_path(location)
         with file_path.open("r", encoding="utf8") as f:
             for line in _yield_json_lines(f, skip=skip):
                 yield line
 
 
-def write_jsonl(location, lines):
+def write_jsonl(location, lines, append=False, append_new_line=True):
     """Create a .jsonl file and dump contents or write to standard output.
 
     location (unicode / Path): The file path. "-" for writing to stdout.
     lines (list): The JSON-serializable contents of each line.
+    append (bool): Whether or not to append to the location.
+    append_new_line (bool): Whether or not to write a new line before appending
+                            to the file.
     """
     if location == "-":  # writing to stdout
         for line in lines:
             print(json_dumps(line))
     else:
-        file_path = _force_path(location, require_exists=False)
-        with file_path.open("a", encoding="utf-8") as f:
+        mode = "a" if append else "w"
+        file_path = force_path(location, require_exists=False)
+        with file_path.open(mode, encoding="utf-8") as f:
+            if append and append_new_line:
+                f.write("\n")
             for line in lines:
                 f.write(json_dumps(line) + "\n")
 
@@ -128,11 +162,3 @@ def _yield_json_lines(stream, skip=False):
                 continue
             raise ValueError("Invalid JSON on line {}: {}".format(line_no, line))
         line_no += 1
-
-
-def _force_path(location, require_exists=True):
-    if not isinstance(location, Path):
-        location = Path(location)
-    if require_exists and not location.exists():
-        raise ValueError("Can't read file: {}".format(location))
-    return location
